Library IEEE;
use IEEE.STD_Logic_1164.all;

library lpm;
use lpm.lpm_components.all;

entity Memory is
  	port ( 
		ain,
		din						:  in std_logic_vector( 7 downto 0);

		clock,
		fetch_store				:  in std_logic;
 
		system_input			: in std_logic_vector(7 downto 0);
		dout, system_output		: out std_logic_vector(7 downto 0)	
	);
end;

architecture behaviour2 of Memory is

	subtype Cell is std_logic_vector(7 downto 0);

	type decode_state_type is ( ram, io );

	signal decode_state, next_decode_state : decode_state_type;

	signal program_dout, ioregister,
		   input_dout    : Cell;

	signal np_output, din_signbit 	: std_logic_vector( 0 downto 0 );

	signal 	clock_inv, 
		zero			: std_logic;

	signal input_we, ram_we 	: std_logic;

	signal address_7bit		: std_logic_vector( 6 downto 0 );
begin 

	address_7bit <= ain(6 downto 0);
	clock_inv <= not clock;

	-- program ram spans memory locations 0 to $7f ( 7 bits )
	program_ram : component LPM_RAM_DQ
		generic map (
			lpm_width => Cell'length,
			lpm_widthad => 7,
			lpm_file => "np_program.mif"
		)
		port map (
			inclock => clock,
			outclock => clock_inv,
			we  => ram_we,
			address => address_7bit,
			data => din,
			q => program_dout
		);


	---------------------------------------------------
	-- address_decode
	--
	-- do address decoding
	---------------------------------------------------
	address_decode_state : process( ain ) 
	begin
	    	case ain(7) is	
				when '0' =>  -- select program ram
					next_decode_state <= ram;
		
				when '1'  => -- select io
					next_decode_state <= io;
	
				when others =>  -- shouldn't happen...
					next_decode_state <= ram;
		end case;

			--
			-- take care of write enable signals
			--   
			case next_decode_state is
				when ram =>
					ram_we <= fetch_store;
					input_we <= '0';
				when io =>
					ram_we <= '0';
					input_we <= fetch_store;
				when others =>
					ram_we <= '0';
					input_we <= '0';
			end case; 
	end process;

	change_decode_state : process(clock)
	begin
		if clock'event and clock='1' then	
			decode_state <= next_decode_state;
		end if;
	end process;

	address_decode : process( decode_state, program_dout, input_dout )
	begin
		case decode_state is 
			when ram =>
				dout <= program_dout;

			when io =>
				dout <= input_dout;
		end case;
	end process;

	---------------------------------
	-- output latch
	--
	-- decode the output latch address and 
	-- write to it
	---------------------------------
	output_latch : process( clock )
	
	begin
		if clock'event and clock='1' then
			case ain(6 downto 0) is
				when "1111111" =>
					input_dout <= system_input;
					if input_we = '1' then
						system_output <= din;
					end if;
				when others =>
					-- nothing
			end case;
		end if;
	end process;

end behaviour2;

