library ieee;
use ieee.std_logic_1164.all;

package types is
	subtype sp_word is std_logic_vector( 7 downto 0 );
end types;

library ieee;
use ieee.std_logic_1164.all;

use work.types.all;

--library stack_processor;
--use work.stack_processor.all;
--use stack_processor.ir_defs.all;

entity test_system is
	port( 
		clock			: in std_logic;
		reset			: in std_logic;
		system_input 	: in sp_word;
		system_output 	: out sp_word;
		processor_out 	: out sp_word;
		program_counter_out : out sp_word;
		-- debugging signals
		memory_out		: out sp_word;
		memory_address	: out sp_word
	);
end;

architecture structure of test_system is
	component stack_processor
		generic( data_stack_index_width : integer := 3; 
			  data_stack_index_depth : integer := 8; 
			  return_stack_index_width : integer := 2; 
			  return_stack_index_depth : integer := 4 
			);
		port (
			reset, clock : in std_logic;
			memdata_in : in std_logic_vector( 7 downto 0);
			memdata_out : out std_logic_vector ( 7 downto 0);
			address_out : out std_logic_vector( 7 downto 0);
			fetch_store : out std_logic;
			topout : out std_logic_vector( 7 downto 0);
			pcout : out std_logic_vector( 7 downto 0)
		);
	end component;
	
	component memory
	  	port ( 
			ain, 
			din, 
			system_input			: in sp_word;
			clock, fetch_store 		: in std_logic;
	
	        dout,
			system_output			: out sp_word
		);
	end component;

	signal  address,
			data_tomem,
			data_tosp,
			periph_tomem,
			program_counter,
			periph_toout			: sp_word;

	signal  fetch_store 
								: std_logic;
	
begin

	system_output <= periph_toout;
	periph_tomem <= system_input;
	program_counter_out <= program_counter;
	memory_out <= data_tosp;
	memory_address <= address;

	my_memory : memory
		port map (
			ain => address,
			din => data_tomem,
			system_input => periph_tomem,
			clock => clock,
			fetch_store => fetch_store,
			dout => data_tosp,
			system_output => periph_toout
		);
	
	my_sp : stack_processor
		generic map (
			data_stack_index_width => 3,
			data_stack_index_depth => 8,
			return_stack_index_width => 2,
			return_stack_index_depth => 4
		)
		port map (
			reset => reset,
			clock => clock,
			memdata_in => data_tosp,
			memdata_out => data_tomem,
			address_out => address,
			fetch_store => fetch_store,
			topout => processor_out,
			pcout => program_counter
		);

end structure;
